package com.democompany.soa.esb.service.util;

import com.democompany.soa.esb.service.util.exception.ValidationException;

/**
 * This is a simple utility class containing basic validation functions. These
 * functions test for simple conditions and throw a validation exception if
 * those conditions are not met. While the functionality provided by these
 * methods is minimal and trivial, it is more elegant to use a one line
 * validate* method in the code than an if statement that may throw an exception
 * (especially in the code that validates a lot of values).
 * 
 * @author Leon Rubin
 */
public class ValidationUtil {
	/**
	 * Validates that the specified string is empty.
	 * 
	 * @param value
	 *            string to test
	 * @param valueName
	 *            name of the string (used in the exception message only to
	 *            specify which value failed validation)
	 * @throws ValidationException
	 *             if the specified string is not empty
	 */
	public static void validateEmpty(String value, String valueName)
			throws ValidationException {
		if (StringUtil.notEmpty(value)) {
			throw new ValidationException(
					"Empty or no value expected for value " + valueName + ", "
							+ value + " found");
		}
	}

	/**
	 * Validates that the specified string is not empty.
	 * 
	 * @param value
	 *            string to test
	 * @param valueName
	 *            name of the string (used in the exception message only to
	 *            specify which value failed validation)
	 * @throws ValidationException
	 *             if the specified string is empty
	 */
	public static void validateNotEmpty(String value, String valueName)
			throws ValidationException {
		if (StringUtil.isEmpty(value)) {
			throw new ValidationException(
					"Empty or no value specified for value " + valueName);
		}
	}

	/**
	 * Validates that the specified integer represents an empty positive integer
	 * value. Since primitive integers always have a value, -1 can be used to
	 * represent a positive integer with a null value.
	 * 
	 * @param value
	 *            integer to test
	 * @param valueName
	 *            name of the integer (used in the exception message only to
	 *            specify which value failed validation)
	 * @throws ValidationException
	 *             if the specified integer has a valid value assigned
	 */
	public static void validateEmptyPositiveInteger(int value, String valueName)
			throws ValidationException {
		if (value > -1) {
			throw new ValidationException("No value expected for value "
					+ valueName + ", " + value + " found");
		}
	}

	/**
	 * Validates that the specified integer represents an non empty positive
	 * integer value. Since primitive integers always have a value, -1 can be
	 * used to represent a positive integer with a null value.
	 * 
	 * @param value
	 *            integer to test
	 * @param valueName
	 *            name of the integer (used in the exception message only to
	 *            specify which value failed validation)
	 * @throws ValidationException
	 *             if the specified integer does not have a valid value assigned
	 */
	public static void validateNotEmptyPositiveInteger(int value,
			String valueName) throws ValidationException {
		if (value < 0) {
			throw new ValidationException("No value specified for value "
					+ valueName);
		}
	}
}
