package com.democompany.soa.esb.service.util.customj;

import java.util.Iterator;

import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQLog;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQPart;
import com.sonicsw.xq.XQService;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;

/**
 * This abstract class defines a template that all Custom Java Services should
 * follow. Unless a service has some unique requirements, it is concerned mainly
 * with processing messages and it's parts. The service code that is generated
 * by the wizard is necessary but contains a lot of framwork code that is not
 * necessary for most developers to see. This class implements a template
 * pattern and contains all that XQ framework code. The child classes simply
 * provide implementation for the processMessage and processPart methods.
 * 
 * @author Leon Rubin
 */
public abstract class AbstractXQService implements XQService {
	private String name;
	private XQInitContext initialContext;

	protected XQLog m_xqLog;

	/**
	 * Creates an abstract service.
	 * 
	 * @param name
	 *            service name
	 */
	public AbstractXQService(String name) {
		this.name = name;
	}

	/**
	 * Initialize the XQService by processing its initialization parameters.
	 * This method implements a required XQService method.
	 * 
	 * @param initialContext
	 *            The Initial Service Context provides access to:<br>
	 *            <ul>
	 *            <li>The configuration parameters for this instance of the
	 *            service</li>
	 *            <li>The XQLog for this instance of the service</li>
	 *            </ul>
	 * @exception XQServiceException
	 *                Used in the event of some error.
	 */
	public void init(XQInitContext initialContext) throws XQServiceException {
		this.initialContext = initialContext;
		m_xqLog = initialContext.getLog();

		m_xqLog.logInformation("[" + name + "] Initializing ...");
	}

	/**
	 * This method allows the service developers to specify the message
	 * processing logic.
	 * 
	 * @param ctx
	 *            XQ service context
	 * @param env
	 *            XQ envelope
	 * @throws XQServiceException
	 *             if a service exception occurs
	 * @throws XQMessageException
	 *             if a message exception occurs
	 */
	protected void processMessage(XQServiceContext ctx, XQEnvelope env)
			throws XQServiceException, XQMessageException {

	}

	/**
	 * This method allows the service developers to specify the part processing
	 * logic.
	 * 
	 * @param ctx
	 *            XQ service context
	 * @param env
	 *            XQ envelope
	 * @param msg
	 *            XQ messaage that is being processed
	 * @param part
	 *            XQ part that is being processed
	 * @param partIndex
	 *            index of the part that is being processed
	 * @throws XQServiceException
	 *             if a service exception occurs
	 * @throws XQMessageException
	 *             if a message exception occurs
	 */
	protected void processPart(XQServiceContext ctx, XQEnvelope env,
			XQMessage msg, XQPart part, int partIndex)
			throws XQServiceException, XQMessageException {

	}

	/**
	 * Handle the arrival of XQMessages in the INBOX, delegate the processing of
	 * the message to the processMessage method and processing of parts to the
	 * processPart method.
	 * 
	 * <p>
	 * This method implement a required XQService method.
	 * 
	 * @param ctx
	 *            The service context.
	 * @exception XQServiceException
	 *                Thrown in the event of a processing error.
	 */
	public void service(XQServiceContext ctx) throws XQServiceException {
		if (ctx == null)
			throw new XQServiceException("Service Context cannot be null.");
		else {
			while (ctx.hasNextIncoming()) {
				XQEnvelope env = ctx.getNextIncoming();

				if (env != null) {
					try {
						m_xqLog.logInformation("[" + name
								+ "] Processing message ...");
						processMessage(ctx, env);

						XQMessage msg = env.getMessage();

						if (msg.getPartCount() > 0) {
							m_xqLog.logInformation("[" + name
									+ "] Processing message parts...");

							for (int i = 0; i < msg.getPartCount(); i++) {
								XQPart prt = msg.getPart(i);

								m_xqLog.logInformation("[" + name
										+ "] Processing message part " + i);
								processPart(ctx, env, msg, prt, i);
							}
						} else {
							m_xqLog.logInformation("[" + name
									+ "] Processing message with no parts...");
							processPartless(ctx, env, msg);
						}
					} catch (XQMessageException me) {
						throw new XQServiceException(
								"Exception accessing XQMessage: "
										+ me.getMessage(), me);
					}

					// Pass message onto the outbox.
					Iterator addressList = env.getAddresses();

					if (addressList.hasNext()) {
						// Add the message to the Outbox
						ctx.addOutgoing(env);
					}
				}
			}
		}
	}

	/**
	 * This method allows the service developers to specify the logic for
	 * processing messages with no parts.
	 * 
	 * @param ctx
	 *            XQ service context
	 * @param env
	 *            XQ envelope
	 * @param msg
	 *            XQ messaage that is being processed
	 * @param part
	 *            XQ part that is being processed
	 * @param partIndex
	 *            index of the part that is being processed
	 * @throws XQServiceException
	 *             if a service exception occurs
	 * @throws XQMessageException
	 *             if a message exception occurs
	 */
	protected void processPartless(XQServiceContext ctx, XQEnvelope env,
			XQMessage msg) throws XQServiceException, XQMessageException {

	}

	/**
	 * Get the reference to the previously saved initial context.
	 * 
	 * @return Initial context reference
	 */
	public XQInitContext getInitialContext() {
		return initialContext;
	}

	/**
	 * Gets the reference to the log object.
	 * 
	 * @return XQLog reference
	 */
	public XQLog getLog() {
		return m_xqLog;
	}

	/**
	 * Gets the service name.
	 * 
	 * @return String containing the service name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Clean up and get ready to destroy the service. This method implement a
	 * required XQService method.
	 */
	public void destroy() {

	}
}
