package com.democompany.soa.esb.service.util.customj;

import com.democompany.soa.esb.service.util.customj.parammap.Parameter;
import com.democompany.soa.esb.service.util.customj.parammap.ParameterConfig;
import com.democompany.soa.esb.service.util.customj.parammap.ParameterMapping;
import com.democompany.soa.esb.service.util.customj.parammap.Parameters;
import com.democompany.soa.esb.service.util.exception.ValidationException;
import com.democompany.soa.esb.service.util.exception.XMLException;
import com.democompany.soa.esb.service.util.xml.XMLParserFactory;
import com.sonicsw.xq.XQConstants;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;

/**
 * This abstract class can serve as a base class for any service that needs to
 * map input and output parameters to XQParts, JMS properties, and literal
 * values. Such service requires at least one initialization parameter called
 * <code>parameterMapping</code> which specifies what input and output
 * parameters are going to be used and what the mapping is. This class is
 * responsible for evaluating input parameters (extracting their values from
 * various message parts) and for updating the appropriate message parts with
 * the value of output parameters. The subclasses get their inputs and outputs
 * through the instance of the <code>Parameters</code> class that is passed
 * into the abstract processRequest method.
 * 
 * @author Leon Rubin
 */
public abstract class XQServiceWithParameterMapping extends AbstractXQService {
	private static final String PARAMETER_MAPPING_FILE_PARAMETER = "parameterMappingFile";
	private static final String PARAMETER_MAPPING_PARSER_CONFIG = "ParameterMappingParser.xml";

	private ParameterMapping parameterMapping;

	/**
	 * Constructor for a XQServiceWithParameterMapping
	 */
	public XQServiceWithParameterMapping(String serviceName) {
		super(serviceName);
	}

	/**
	 * Initialize the service and read initialization parameters. This service
	 * uses a single parameter called <code>parameterMappingFile</code> that
	 * specifies the location of the parameter configuration file used to map
	 * input and output parameters to various parts of the XQMessage.
	 * 
	 * @param initialContext
	 *            initial context that provides access to configuration
	 *            parameters and access to other objects important during
	 *            initialization
	 * @exception XQServiceException
	 *                thrown in the event of some error.
	 */
	public void init(XQInitContext initialContext) throws XQServiceException {
		super.init(initialContext);

		String parameterMappingFile = initialContext.getParameters()
				.getParameter(PARAMETER_MAPPING_FILE_PARAMETER,
						XQConstants.PARAM_STRING);
		m_xqLog.logInformation("Reading parameter mapping file "
				+ parameterMappingFile);

		try {
			parameterMapping = (ParameterMapping) XMLParserFactory.getParser()
					.parse(parameterMappingFile,
							PARAMETER_MAPPING_PARSER_CONFIG);
		} catch (XMLException e) {
			throw new XQServiceException(
					"Failed to parse XML configuration file", e);
		}

		m_xqLog.logInformation("Initialized with the following information "
				+ parameterMapping);
	}

	/**
	 * This class processes the message by evaluating input parameters, creating
	 * blank instances of output parameters, and calling the abstract
	 * processRequest method. It is expected that the subclassing service will
	 * execute some logic based on the value of input parameters and will set
	 * the values of output parameters. After the processRequest method returns,
	 * the message will be updated to the values contained in output parameters.
	 */
	protected void processMessage(XQServiceContext ctx, XQEnvelope env)
			throws XQServiceException, XQMessageException {
		XQMessage msg = env.getMessage();
		Parameters parameters = new Parameters();

		try {
			for (ParameterConfig inputParameterConfig : parameterMapping
					.getInputParameterConfigs()) {
				parameters
						.addInputParameter(inputParameterConfig.evaluate(msg));
			}

			for (ParameterConfig outputParameterConfig : parameterMapping
					.getOutputParameterConfigs()) {
				parameters.addOutputParameter(outputParameterConfig
						.createParameter());
			}

			processRequest(ctx, env, parameters);

			for (Parameter outputParameter : parameters.getOutputParameters()) {
				ParameterConfig outputParameterConfig = parameterMapping
						.getOutputParameterConfig(outputParameter.getName());

				outputParameterConfig.update(msg, outputParameter);
			}
		} catch (ValidationException e) {
			throw new XQServiceException(
					"Parameter mapping configuration failed validation", e);
		}
	}

	/**
	 * This method allows subclasses to specify the processing logic for the
	 * request. All service inputs and outputs are accessed through the
	 * <code>parameters</code> object. The inputs are already set but the
	 * subclassing services are exprected to calculate and set the value of
	 * output parameters.
	 * 
	 * @param ctx
	 *            service context
	 * @param env
	 *            envelope
	 * @param parameters
	 *            object containing all input and output parameters for this
	 *            request
	 * @throws XQServiceException
	 *             if a problem occurs during the request processing
	 */
	protected abstract void processRequest(XQServiceContext ctx,
			XQEnvelope env, Parameters parameters) throws XQServiceException;
}