package com.democompany.soa.esb.service.util.customj.parammap;

import com.democompany.soa.esb.service.util.StringUtil;
import com.democompany.soa.esb.service.util.ValidationUtil;
import com.democompany.soa.esb.service.util.exception.ValidationException;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQPart;

/**
 * This class represents a parameter configuration from a parameter mapping file
 * (describes how to map a single service parameter to a message compartment). A
 * message compartment is any part of the message where data can be stored
 * (XQPart in multipart messages, JMS property, etc).
 * 
 * @author Leon Rubin
 */
public class ParameterConfig extends Parameter {
	private static final String JMS_PROPERTY_COMPARTMENT_TYPE = "JMSProperty";
	private static final String XQ_PART_COMPARTMENT_TYPE = "XQPart";

	private String compartmentName;
	private String compartmentType;
	private int compartmentIndex;

	/**
	 * Creates parameter configuration object.
	 */
	public ParameterConfig() {
		compartmentIndex = -1;
	}

	/**
	 * Gets the compartment name. Depending on compartment type, the name is
	 * either the content Id of the XQPart or the JMS property name. This
	 * attribute is optional since it is not applicable when the parameter has a
	 * literal value.
	 * 
	 * @return String containing the compartment name
	 */
	public String getCompartmentName() {
		return compartmentName;
	}

	/**
	 * Sets the compartment name.
	 * 
	 * @param compartmentName
	 *            compartment name
	 */
	public void setCompartmentName(String compartmentName) {
		this.compartmentName = compartmentName;
	}

	/**
	 * Gets the compartment type (XQPart or the JMS property). This attribute is
	 * optional since it is not applicable when the parameter has a literal
	 * value.
	 * 
	 * @return String containing the compartment type
	 */
	public String getCompartmentType() {
		return compartmentType;
	}

	/**
	 * Sets the compartment type.
	 * 
	 * @param compartmentType
	 *            compartment type
	 */
	public void setCompartmentType(String compartmentType) {
		this.compartmentType = compartmentType;
	}

	/**
	 * Gets the compartment index. This attribute is optional since it is only
	 * applicable for parameters with compartment type XQPart. In that case the
	 * index is the index of part that contains the value.
	 * 
	 * @return integer containing the compartment index
	 */
	public int getCompartmentIndex() {
		return compartmentIndex;
	}

	/**
	 * Sets the compartment index.
	 * 
	 * @param compartmentIndex
	 *            compartment index
	 */
	public void setCompartmentIndex(int compartmentIndex) {
		this.compartmentIndex = compartmentIndex;
	}

	/**
	 * Evaluates the parameter value based on the data present in the specified
	 * message and the mapping configuration for this parameter. Parameters with
	 * literal value require no further evaluation. Parameters with compartment
	 * type XQPart get their value from the content of the part. The part is
	 * addressed by the content Id or part index Parameters with compartment
	 * type JMSProperty get their value from the property value.
	 * 
	 * @param msg
	 *            message containing the input data (in XQParts and JMS
	 *            properties)
	 * @return Parameter representing an input parameter described by this
	 *         parameter config and complete with a value
	 * @throws ValidationException
	 *             if any problems are discovered with the parameter
	 *             configuration
	 * @throws XQMessageException
	 *             if a problem occurs while extracting information from the XQ
	 *             message
	 */
	public Parameter evaluate(XQMessage msg) throws ValidationException,
			XQMessageException {
		Parameter parameter = createParameter();

		// name is always required
		ValidationUtil.validateNotEmpty(getName(), "name");
		// type is always required
		ValidationUtil.validateNotEmpty(getType(), "type");

		// if value is not empty then no further evaluation is necessary because
		// the parameter is initialized with a literal value
		if (StringUtil.isEmpty(getValue())) {
			// if value is empty then the parameter must specify where to get it
			// (compartment information)
			if (compartmentType.equals(JMS_PROPERTY_COMPARTMENT_TYPE)) {
				// if compartment type is JMS property, then the compartment
				// name must specify the property name
				ValidationUtil.validateNotEmpty(compartmentName,
						"compartmentName");

				parameter.setValue(msg.getStringHeader(compartmentName));
			} else if (compartmentType.equals(XQ_PART_COMPARTMENT_TYPE)) {
				// if compartment type is XQ part, then the part can be
				// specified either by name or index

				if (StringUtil.isEmpty(compartmentName)) {
					// since compartment name is not specified, the compartment
					// index is required
					ValidationUtil.validateNotEmptyPositiveInteger(
							compartmentIndex, "compartmentIndex");

					parameter.setValue((String) msg.getPart(compartmentIndex)
							.getContent());
				} else {
					// compartment name is specified so get the value from
					// compartment with that name
					parameter.setValue((String) msg.getPart(compartmentName)
							.getContent());
					// since the compartment name is specified, the compartment
					// index
					// should not be specified
					ValidationUtil.validateEmptyPositiveInteger(
							compartmentIndex, "compartmentIndex");
				}
			} else {
				throw new ValidationException("Invalid compartment type "
						+ compartmentType);
			}
		} else {
			// if a literal value is specified then compartment attributes are
			// not applicable and should not be specified
			ValidationUtil.validateEmpty(compartmentName, "compartmentName");
			ValidationUtil.validateEmpty(compartmentType, "compartmentType");
			ValidationUtil.validateEmptyPositiveInteger(compartmentIndex,
					"compartmentIndex");
		}

		return parameter;
	}

	/**
	 * Creates a blank parameter object using this parameter configuration as a
	 * template.
	 * 
	 * @return Parameter object with the name, type, and value copied from the
	 *         parameter configuration. Unless the value is literal, it will be
	 *         empty
	 */
	public Parameter createParameter() {
		Parameter parameter = new Parameter();

		parameter.setName(getName());
		parameter.setType(getType());
		parameter.setValue(getValue());

		return parameter;
	}

	/**
	 * Update the message with the value of the output parameter. If the
	 * parameter is mapped to an XQPart, the parameter value will be placed in a
	 * new XQPart with the contentId equal to the parameter name. If the
	 * parameter is mapped to a JMS property, the property with the same name as
	 * the parameter name will be set to parameter value.
	 * 
	 * @param msg
	 *            message where the output parameter value should be written
	 * @param parameter
	 *            output parameter containing the value to be written
	 * @throws ValidationException
	 *             if any problems are discovered with the parameter
	 *             configuration
	 * @throws XQMessageException
	 *             if a problem occurs while writing information to the XQ
	 *             message
	 */
	public void update(XQMessage msg, Parameter parameter)
			throws ValidationException, XQMessageException {
		// name is always required
		ValidationUtil.validateNotEmpty(getName(), "name");

		// compartment type must be specified
		ValidationUtil.validateNotEmpty(compartmentType, "compartmentType");
		// compartment name must be specified
		ValidationUtil.validateNotEmpty(compartmentName, "compartmentName");

		if (compartmentType.equals(JMS_PROPERTY_COMPARTMENT_TYPE)) {
			msg.setStringHeader(compartmentName, getValue());
		} else if (compartmentType.equals(XQ_PART_COMPARTMENT_TYPE)) {
			XQPart part = msg.createPart();

			part.setContentId(compartmentName);
			part.setContent(parameter.getValue(), "text");

			msg.addPart(part);
		} else {
			throw new ValidationException("Invalid compartment type "
					+ compartmentType);
		}
	}

	public String toString() {
		return "{" + "name=" + getName() + ", type=" + getType() + ", value="
				+ getValue() + ", compartmentName=" + compartmentName
				+ ", compartmentType=" + compartmentType
				+ ", compartmentIndex=" + compartmentIndex + "}";
	}
}
