package com.democompany.soa.esb.service.xquery.customj;

import javax.xml.namespace.QName;

import net.sf.saxon.javax.xml.xquery.XQConnection;
import net.sf.saxon.javax.xml.xquery.XQDataSource;
import net.sf.saxon.javax.xml.xquery.XQException;
import net.sf.saxon.javax.xml.xquery.XQItem;
import net.sf.saxon.javax.xml.xquery.XQPreparedExpression;
import net.sf.saxon.javax.xml.xquery.XQResultSequence;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.democompany.soa.esb.service.util.customj.XQServiceWithParameterMapping;
import com.democompany.soa.esb.service.util.customj.parammap.Parameter;
import com.democompany.soa.esb.service.util.customj.parammap.Parameters;
import com.democompany.soa.esb.service.util.exception.XMLException;
import com.democompany.soa.esb.service.util.xml.XMLParserFactory;
import com.democompany.soa.esb.service.xquery.customj.config.XQQuery;
import com.democompany.soa.esb.service.xquery.customj.config.XQueryServiceConfig;
import com.sonicsw.xq.XQConstants;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQParameters;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;

/**
 * This is the custom Java service that implements the XQuery functinality.
 * XQuery is a new query language for XML data. It provides functions for
 * gathering XML from variety of sources including files, URLs, etc and
 * performing SQL style operations on them. The XQueryService integrates XQuery
 * API with the ESB and allows it to feed parameters from and write output to
 * various parts of the JMS/XQ message. The mapping between parameters and
 * message parts is defined in a separate parameter mapping XML config. The
 * mapping logic is inherited from the parent class.
 * 
 * @author Leon Rubin
 */
public class XQueryService extends XQServiceWithParameterMapping {
	private static final String CONFIG_FILE_PARAMETER = "configFile";
	private static final String XQJ_IMPL_BEAN = "xqueryImpl";
	private static final String SPRING_CONFIG = "XQueryServiceBeans.xml";
	private static final String XML_PARAMETER_TYPE = "xml";
	private static final String XQUERY_SERVICE_PARSER_CONFIG = "XQueryServiceConfigParser.xml";

	private XQueryServiceConfig config;
	private XQConnection connection;

	/**
	 * Constructor for a XQueryService
	 */
	public XQueryService() {
		super("XQueryService");
	}

	/**
	 * Initialize the service and read initialization parameters. This service
	 * uses a single parameter called <code>configFile</code> that specifies
	 * the location of the configuration file used to configure the xquery
	 * service.
	 * 
	 * @param initialContext
	 *            initial context that provides access to configuration
	 *            parameters and access to other objects important during
	 *            initialization
	 * @exception XQServiceException
	 *                thrown in the event of some error.
	 */
	public void init(XQInitContext initialContext) throws XQServiceException {
		super.init(initialContext);

		XQParameters params = initialContext.getParameters();

		String configFile = params.getParameter(CONFIG_FILE_PARAMETER,
				XQConstants.PARAM_STRING);
		m_xqLog.logInformation("Reading configuration file " + configFile);

		try {
			config = (XQueryServiceConfig) XMLParserFactory.getParser().parse(
					configFile, XQUERY_SERVICE_PARSER_CONFIG);

			m_xqLog
					.logInformation("Initialized with the following information "
							+ config);

			ApplicationContext context = new ClassPathXmlApplicationContext(
					SPRING_CONFIG);
			XQDataSource ds = (XQDataSource) context.getBean(XQJ_IMPL_BEAN);

			connection = ds.getConnection();
		} catch (XQException e) {
			throw new XQServiceException(
					"Failed to establish XQuery connection", e);
		} catch (XMLException e) {
			throw new XQServiceException(
					"Failed to parse XQuery service configuration file", e);
		}
	}

	/**
	 * Processes the service request by executing the queries configured in the
	 * XQuery XML configuration file against one or more XML documents contained
	 * in various parts of the XQ message.
	 */
	@Override
	protected void processRequest(XQServiceContext ctx, XQEnvelope env,
			Parameters parameters) throws XQServiceException {
		try {
			for (XQQuery query : config.getQueries()) {
				XQPreparedExpression queryExpression = prepareQueryAndBindParameters(
						query.getExpression(), parameters);
				Parameter outputParameter = parameters.getOutputParameter(query
						.getOutputParameterName());

				outputParameter.setValue(getQueryResult(queryExpression));
			}
		} catch (XQException e) {
			throw new XQServiceException(e);
		}
	}

	/**
	 * Executes the XQuery prepared expression and gets the result as an XML
	 * string.
	 * 
	 * @param queryExpression
	 *            prepared XQuery expression to execute
	 * @return String containing the resulting XML
	 * @throws XQException
	 *             if a problem occurs during the XQuery operation
	 */
	private String getQueryResult(XQPreparedExpression queryExpression)
			throws XQException {
		XQResultSequence result = queryExpression.executeQuery();

		StringBuilder builder = new StringBuilder();

		while (result.next()) {
			XQItem item = result.getItem();
			builder.append(item.getItemAsString(null));
			builder.append('\n');
		}

		return builder.toString();
	}

	/**
	 * Prepares an XQuery expression object from a string containing the query
	 * statement and binds input parameters.
	 * 
	 * @param query
	 *            string containing the XQuery expression
	 * @param parameters
	 *            a collection of input and output parameters (only input
	 *            parameters are used by this method)
	 * @return prepared XQ expression ready to be executed
	 * @throws XQException
	 */
	private XQPreparedExpression prepareQueryAndBindParameters(String query,
			Parameters parameters) throws XQException {
		XQPreparedExpression queryExpression = connection
				.prepareExpression(query);

		for (Parameter inputParameter : parameters.getInputParameters()) {
			if (query.indexOf('$' + inputParameter.getName()) > -1) {
				QName inputParameterQName = new QName(inputParameter.getName());

				if (inputParameter.getType().equalsIgnoreCase(
						XML_PARAMETER_TYPE)) {
					queryExpression.bindDocument(inputParameterQName,
							inputParameter.getValue(), null);
				} else {
					queryExpression.bindObject(inputParameterQName,
							inputParameter.getValue(), null);
				}
			}
		}

		return queryExpression;
	}
}